
' USB descriptors for a HID device
'USBMEMORYADDRESS Con	$400	' USB RAM starts here (set in device header file)
USBMEMORYSIZE	Con	256	' USB RAM size in bytes
USBReservedMemory Var Byte[USBMEMORYSIZE] USBMEMORYADDRESS	' Reserve memory used by USB assembler code

goto	hid_desc_end	' Skip over all of the USB assembler code
asm

#define	USB_EP0_BUFF_SIZE 	64	; 8, 16, 32, or 64
#define	USB_MAX_NUM_INT		1	; For tracking Alternate Setting
#define	USB_MAX_EP_NUMBER  	1	; UEP1
#define	NUM_CONFIGURATIONS	1
#define	NUM_INTERFACES		1

#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_FULL_SPEED|USB_PING_PONG__NO_PING_PONG
;#define UCFG_VAL	USB_PULLUP_ENABLE|USB_INTERNAL_TRANSCEIVER|USB_LOW_SPEED|USB_PING_PONG__NO_PING_PONG

;#define USE_SELF_POWER_SENSE_IO
;#define USE_USB_BUS_SENSE_IO

#define USB_POLLING

; HID
; Endpoints Allocation
#define	HID_INTF_ID		  0x00
#define	HID_EP			  1
#define	HID_INT_OUT_EP_SIZE	  64
#define	HID_INT_IN_EP_SIZE	  64
#define	HID_NUM_OF_DSC		  1

   include	"usb_hid.asm"	; Include rest of USB files, starting with HID class code

; ******************************************************************
; This table is polled by the host immediately after USB Reset has been released.
; This table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application DEPENDENT. Modify these to meet
; your specifications.
; ******************************************************************
DeviceDescriptor
	retlw	(EndDeviceDescriptor-DeviceDescriptor)/2	; bLength Length of this descriptor
	retlw	USB_DESCRIPTOR_DEVICE ; bDescType This is a DEVICE descriptor
	retlw	0x10		; bcdUSBUSB Revision 1.10 (low byte)
	retlw	0x01		; high byte
	retlw	0x00		; bDeviceClass zero means each interface operates independently
	retlw	0x00		; bDeviceSubClass
	retlw	0x00		; bDeviceProtocol 
	retlw	USB_EP0_BUFF_SIZE ; bMaxPacketSize for EP0

        ; idVendor (low byte, high byte)
	retlw	0x61
	retlw	0x02

        ; idProduct (low byte, high byte)
	retlw	0xDD
	retlw	0x07

        retlw	0x00		; bcdDevice (low byte)
	retlw	0x00		; (high byte)
	retlw	0x01		; iManufacturer (string index)
	retlw	0x02		; iProduct      (string index)

        ; iSerialNumber (string index)
	retlw	0x03
	retlw	NUM_CONFIGURATIONS ; bNumConfigurations
EndDeviceDescriptor

; ******************************************************************
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application DEPENDENT. 
; Modify these to meet your specifications.
; ******************************************************************
; Configuration pointer table
USB_CD_Ptr
Configs
	db	low Config1, high Config1
	db	upper Config1, 0

; Configuration Descriptor
Config1
	retlw	(Interface1-Config1)/2	; bLength Length of this descriptor
	retlw	USB_DESCRIPTOR_CONFIGURATION ; bDescType 2=CONFIGURATION
Config1Len
	retlw	low ((EndConfig1 - Config1)/2)	; Length of this configuration
	retlw	high ((EndConfig1 - Config1)/2)
	retlw	0x01		; bNumInterfaces Number of interfaces
	retlw	0x01		; bConfigValue Configuration Value
	retlw	0x00		; iConfig (string index)
	retlw	_DEFAULT|_SELF	; bmAttributes attributes - bus powered

        ; Max power consumption (2X mA)
	retlw	0x32
Interface1
	retlw	(HIDDescriptor1-Interface1)/2	; length of descriptor
	retlw	USB_DESCRIPTOR_INTERFACE
	retlw	0x00		; number of interface, 0 based array
	retlw	0x00		; alternate setting
	retlw	0x02		; number of endpoints used in this interface
	retlw	0x03		; interface class - assigned by the USB
	retlw	0x00		; boot device
	retlw	0x01		; interface protocol
	retlw 	0x00		; index to string descriptor that describes this interface
HIDDescriptor1
	retlw	(Endpoint1In-HIDDescriptor1)/2	; descriptor size (9 bytes)
        retlw	DSC_HID		; descriptor type (HID)
	retlw	0x11		; HID class release number (1.11)
	retlw	0x01
        retlw	0x00		; Localized country code (none)
        retlw	0x01		; # of HID class descriptor to follow (1)
        retlw	0x22		; Report descriptor type (HID)
ReportDescriptor1Len
	retlw	low ((EndReportDescriptor1-ReportDescriptor1)/2)
	retlw	high ((EndReportDescriptor1-ReportDescriptor1)/2)
Endpoint1In
	retlw	(EndPoint1Out-Endpoint1In)/2	; length of descriptor
	retlw	USB_DESCRIPTOR_ENDPOINT
	retlw	HID_EP|_EP_IN		; EP1, In
	retlw	_INT		; Interrupt
	retlw	low (HID_INT_IN_EP_SIZE)		; This should be the size of the endpoint buffer
	retlw	high (HID_INT_IN_EP_SIZE)
	retlw	0x0A                        ; Polling interval
EndPoint1Out
	retlw	(EndConfig1-EndPoint1Out)/2	; Length of this Endpoint Descriptor
	retlw	USB_DESCRIPTOR_ENDPOINT		; bDescriptorType = 5 for Endpoint Descriptor
	retlw	HID_EP|_EP_OUT		; Endpoint number & direction
	retlw	_INT		; Transfer type supported by this Endpoint
	retlw	low (HID_INT_OUT_EP_SIZE)		; This should be the size of the endpoint buffer
	retlw	high (HID_INT_OUT_EP_SIZE)
	retlw	0x0A                        ; Polling interval
EndConfig1

ReportDescriptor1
    retlw 0x05 
    retlw 0x01                    ; USAGE_PAGE (Generic Desktop)
    
    retlw 0x09 
    retlw 0x06                    ; USAGE (Keyboard)
    
    retlw 0xa1
    retlw 0x01                    ; COLLECTION (Application)
    
    retlw 0x05
    retlw 0x07                    ;   USAGE_PAGE (Keyboard)
    
    retlw 0x19
    retlw 0xe0                    ;   USAGE_MINIMUM (Keyboard LeftControl)
    
    retlw 0x29
    retlw 0xe7                    ;   USAGE_MAXIMUM (Keyboard Right GUI)
    
    retlw 0x15
    retlw 0x00                    ;   LOGICAL_MINIMUM (0)
    
    retlw 0x25
    retlw 0x01                    ;   LOGICAL_MAXIMUM (1)
    
    retlw 0x75
    retlw 0x01                    ;   REPORT_SIZE (1)
    
    retlw 0x95
    retlw 0x08                    ;   REPORT_COUNT (8)
    
    retlw 0x81
    retlw 0x02                    ;   INPUT (DataVarAbs)
    
    retlw 0x95
    retlw 0x01                    ;   REPORT_COUNT (1)
    
    retlw 0x75
    retlw 0x08                    ;   REPORT_SIZE (8)
    
    retlw 0x81
    retlw 0x03                    ;   INPUT (CnstVarAbs)
    
    retlw 0x95
    retlw 0x05                    ;   REPORT_COUNT (5)
    
    retlw 0x75
    retlw 0x01                    ;   REPORT_SIZE (1)
    
    retlw 0x05
    retlw 0x08                    ;   USAGE_PAGE (LEDs)
    
    retlw 0x19
    retlw 0x01                    ;   USAGE_MINIMUM (Num Lock)
    
    retlw 0x29
    retlw 0x05                    ;   USAGE_MAXIMUM (Kana)
    
    retlw 0x91
    retlw 0x02                    ;   OUTPUT (DataVarAbs)
    
    retlw 0x95
    retlw 0x01                    ;   REPORT_COUNT (1)
    
    retlw 0x75
    retlw 0x03                    ;   REPORT_SIZE (3)
    
    retlw 0x91
    retlw 0x03                    ;   OUTPUT (CnstVarAbs)
    
    retlw 0x95
    retlw 0x06                    ;   REPORT_COUNT (6)
    
    retlw 0x75
    retlw 0x08                    ;   REPORT_SIZE (8)
    
    retlw 0x15
    retlw 0x00                    ;   LOGICAL_MINIMUM (0)
    
    retlw 0x25
    retlw 0x65                    ;   LOGICAL_MAXIMUM (101)
    
    retlw 0x05
    retlw 0x07                    ;   USAGE_PAGE (Keyboard)
    
    retlw 0x19
    retlw 0x00                    ;   USAGE_MINIMUM (Reserved (no event indicated))
    
    retlw 0x29
    retlw 0x65                    ;   USAGE_MAXIMUM (Keyboard Application)
    
    retlw 0x81
    retlw 0x00                    ;   INPUT (DataAryAbs)
    
    retlw 0xc0                          ; End Collection
EndReportDescriptor1

; String pointer table
USB_SD_Ptr
Strings
	db	low String0, high String0
        db	upper String0, 0
	db	low String1, high String1
        db	upper String1, 0
	db	low String2, high String2
       	db	upper String2, 0
	db	low String3, high String3
	db	upper String3, 0

String0
	retlw	(String1-String0)/2	; Length of string
	retlw	USB_DESCRIPTOR_STRING   ; Descriptor type 3
	retlw	0x09		        ; Language ID (as defined by MS 0x0409)
	retlw	0x04

; company name
String1
	retlw	(String2-String1)/2
	retlw	USB_DESCRIPTOR_STRING
	
        retlw   'D'
        retlw   0x00
        retlw   'L'
        retlw   0x00
        retlw   'C'
        retlw   0x00
        retlw   'i'
        retlw   0x00
        retlw   'n'
        retlw   0x00
        retlw   'c'
        retlw   0x00

	
; product name	
String2
	retlw	(String3-String2)/2
	retlw	USB_DESCRIPTOR_STRING
	
        retlw   'C'
        retlw   0x00
        retlw   'D'
        retlw   0x00
        retlw   'U'
        retlw   0x00
        retlw   '_'
        retlw   0x00
        retlw   'K'
        retlw   0x00
        retlw   'e'
        retlw   0x00
        retlw   'y'
        retlw   0x00
        retlw   'B'
        retlw   0x00
        retlw   'o'
        retlw   0x00
        retlw   'a'
        retlw   0x00
        retlw   'r'
        retlw   0x00
        retlw   'd'
        retlw   0x00


; serial number
String3
	retlw	(String4-String3)/2
	retlw	USB_DESCRIPTOR_STRING
	
        retlw   '1'
        retlw   0x00
        retlw   '0'
        retlw   0x00
        retlw   '0'
        retlw   0x00
        retlw   '1'
        retlw   0x00

String4


endasm
hid_desc_end
 
 
